_G.STM = _G.STM or {}
STM.trip_mines = {}
STM.cam_destroyed = false
STM.pager_answering = false
STM.jumping = {host = 'undef', husk = 'undef'}
STM.ducking = {husk = 'undef'}
STM._path = ModPath
STM._settings = {}

function STM:load()
	local file = io.open(SavePath .. 'stealth_mutators.txt', 'r')
	if file then
		STM._settings = json.decode(file:read('*all'))
		file:close()
		if STM._settings.version == 1 then
			STM._settings.players = {}
			STM._settings.crouch_jump_realism = false
			STM._settings.version = 2
			STM:save()
		end
	else
		STM._settings.version = 2
		STM._settings.security_room_global = true
		STM._settings.radio_channel_single = true
		STM._settings.pagers_alarm = 5
		STM._settings.civilian_penalty = 5
		STM._settings.trip_mines_lifetime = 90
		STM._settings.crouch_jump_realism = true
		STM._settings.players = {}
		STM:save()
	end
end

function STM:save()
	local file = io.open(SavePath .. 'stealth_mutators.txt', 'w+')
	if file then
		file:write(json.encode(STM._settings))
		file:close()
	end
end

function STM:set_pagers()
	for i = 4, 1, -1 do
		if i < STM._settings.pagers_alarm then
			tweak_data.player.alarm_pager['bluff_success_chance'][i] = 1
			tweak_data.player.alarm_pager['bluff_success_chance_w_skill'][i] = 1
		else
			tweak_data.player.alarm_pager['bluff_success_chance'][i] = 0
			tweak_data.player.alarm_pager['bluff_success_chance_w_skill'][i] = 0
		end
	end
end

function STM:print_modifiers()
	local message = 'Alarm raised after '
	local modifier_message = {
		['1_pagers'] = '',
		['2_civilians'] = '',
		['3_answering'] = ''
	}
	local modified = false
	if STM._settings.pagers_alarm < 5 then
		modifier_message['1_pagers'] = STM._settings.pagers_alarm - 1 .. ' pagers'
	end
	if STM._settings.civilian_penalty > -1 then
		modifier_message['2_civilians'] = STM._settings.civilian_penalty .. ' dead civs'
	end
	if STM._settings.radio_channel_single then
		modifier_message['3_answering'] = 'pagers answered simultaneously'
	end
	local connector = false
	for _, m in pairs(modifier_message) do
		if m ~= '' then
			if connector then
				message = message .. ' or '
			else
				connector = true
			end
			message = message .. m
			modified = true
		end
	end
	if modified then
		message = message .. '! '
	end
	if STM._settings.security_room_global then
		if modified then
			message = message .. 'Cameras always enabled except on loop!'
		else
			message = 'Cameras always enabled except on loop!'
			modified = true
		end	
	end
	if modified then return message end
	return ''
end

function STM:update_menu(chat_enabled)
	local in_menu = BaseNetworkHandler._gamestate_filter.menu[game_state_machine:current_state_name()]
	local enabled = in_menu and not chat_enabled and not Network:is_client()
	local menu = MenuHelper:GetMenu('stm#27305_menu')
	if menu then
		for _, item in ipairs(menu._items_list) do
			item:set_enabled(enabled)
		end
	end
end

function STM:warn_peers()
	if Global.game_settings.permission == 'friends_only' then return end
	STM:load()
	local new_peer = true
	for i, peer in pairs(managers.network:session():peers()) do
		if table.contains(STM._settings.players, peer:user_id()) then
			new_peer = false
		end
	end
	if not new_peer then return end
	DelayedCalls:Remove('STM#27305_DelayedCalls_warn_peers')
	DelayedCalls:Add('STM#27305_DelayedCalls_warn_peers', 1,
	function()
		for i, peer in pairs(managers.network:session():peers()) do
			STM._settings.players[i-1] = peer:user_id()
		end
		STM:save()
		local peer_local = managers.network:session():local_peer()
		if peer_local then
			local message = STM:print_modifiers()
			if message ~= '' then
				managers.chat:send_message(1, peer_local, message)
			end	
		end
	end)
end

Hooks:AddHook('LocalizationManagerPostInit', 'STM#27305_LocalizationManager_PostInit',
function(self)
	self:load_localization_file(STM._path .. 'loc/en.txt', false)
end)

Hooks:AddHook('MenuManagerInitialize', 'STM#27305_MenuManager_Initialize',
function()
	MenuCallbackHandler.stm_clbk_menu_cameras = function(self, item)
		STM._settings.security_room_global = (item:value() == 'on' and true or false)
		STM:save()
	end
	MenuCallbackHandler.stm_clbk_menu_answering = function(self, item)
		STM._settings.radio_channel_single = (item:value() == 'on' and true or false)
		STM:save()
	end
	MenuCallbackHandler.stm_clbk_menu_pagers = function(self, item)
		STM._settings.pagers_alarm = item:value()
		STM:save()
	end
	MenuCallbackHandler.stm_clbk_menu_civilians = function(self, item)
		STM._settings.civilian_penalty = math.floor(item:value())
		STM:save()
	end
	MenuCallbackHandler.stm_clbk_menu_trip_mines = function(self, item)
		STM._settings.trip_mines_lifetime = math.floor(item:value())
		STM:save()
	end
	MenuCallbackHandler.stm_clbk_menu_detection = function(self, item)
		STM._settings.crouch_jump_realism = (item:value() == 'on' and true or false)
		STM:save()
	end
	MenuCallbackHandler.stm_clbk_menu_reset = function()
		STM._settings.security_room_global = false
		STM._settings.radio_channel_single = false
		STM._settings.pagers_alarm = 5
		STM._settings.civilian_penalty = -1
		STM._settings.trip_mines_lifetime = -1
		STM._settings.crouch_jump_realism = false
		local menu = MenuHelper:GetMenu('stm#27305_menu')
		menu._items_list[1]:set_value('off')
		menu._items_list[2]:set_value('off')
		menu._items_list[3]:set_value(5)
		menu._items_list[4]:set_value(-1)
		menu._items_list[5]:set_value(-1)
		menu._items_list[6]:set_value('off')
		managers.viewport:resolution_changed()
		STM:save()
	end
	STM:load()
	MenuHelper:LoadFromJsonFile(STM._path .. 'menu.txt', STM, STM._settings)
end)

Hooks:AddHook('MenuManagerOnOpenMenu', 'STM#27305_MenuManager_OnOpenMenu',
function()
	if managers.menu_component and managers.menu_component._game_chat_gui then
		STM:update_menu(managers.menu_component._game_chat_gui:enabled())
	end
end)